## RPEP project #########################
## Vandermeulen Sam-academiejaar 22-23 ##
## begeleider: Vincent Hoofs #########

## import modules ##
from psychopy import visual, event, core, gui, data
from psychopy.hardware import keyboard
import pandas, numpy, os
import random
import time
import math

## file management and participant info ##

## set the directory ##
my_directory = os.getcwd()

## construct the name of the folder that will hold the data ##
directory_to_write_to      = my_directory + "/data" 

if not os.path.isdir(directory_to_write_to):
    os.mkdir(directory_to_write_to)

## initialize the participant information dialog box ##
info = {"Participant name":"Incognito", "Participant number":0, "Gender":["male", "female", "third gender"], "Handedness":["right", "left"]}

## make sure the data file has a novel name ##
already_exists = True
print(directory_to_write_to)
while already_exists:
    # present the dialog box #
    myDlg = gui.DlgFromDict(dictionary = info, title = "Exp-RPEP")

    # construct the name of the data file #
    file_name = directory_to_write_to + "/Exp_RPEP_" + str(info["Participant number"])
    # check whether the name of the data file has already been used #
    if not os.path.isfile(file_name + ".csv"):
        # if there isn't a data file with this name used yet, we're ready to start #
        already_exists = False
    else:
        # if the data file name has already been used, ask the participant to inser a different participant number #
        already_exists = True
        myDlg2 = gui.Dlg(title = "Error")
        myDlg2.addText("Try another participant number")
        myDlg2.show()

## extract the name of the participant from the dialog box information ##
subject_name = info["Participant name"]

## remove the name of the participant from the dialog box information ##
info.pop("Participant name")

## start the ExperimentHandler, add the output file name and store the dialog box info ##
thisExp = data.ExperimentHandler(dataFileName = file_name, extraInfo = info)

## welcoming participant ##
###########################
#win_width       = 1536
#win_height      = 864
win             = visual.Window(fullscr=True, units='norm', color = "white")

## messages and stimuli
Welcome         = visual.TextStim(win, text = "Welcome " + subject_name + "!\n\nPress the space bar to continue.", color = "black")
Instructions    = visual.TextStim(win, text = "ok", color = "black")
Block_start     = visual.TextStim(win, text = "ok", color = "black") 

Instructions.text = (   "In this experiment you will be see 12 grating stimuli, presented in a circle.\n" +
                        "Indicate as quickly as possible in which quadrant the target grating stimulus is presented.\n\n" +
                        "the target stimulus is either horizontally oriented (and the distractors vertically oriented) .\n" +
                        "or vice versa. There are four response keys: up, down, left and right.\n\n" +
                        "alert the research leader if there is anything not clear; otherwise.\n" +
                        "Press the space bar to start the experiment.")

Block_start.text = (    "the first block of the experiment will start after you press the space key.\n" +
                        "The experiment contains 21 blocks whith each a number of trials. \n" + 
                        "You will get a 2min break after each block to rest your eyes and wander around. \n" + 
                        "Press space bar to continue.")

## Display the welcome message ##
Welcome.draw()
win.flip()
event.waitKeys(keyList = "space")

## Display the instructions ##
Instructions.draw()
win.flip()
event.waitKeys(keyList = "space")

## showing image ##
image_stim = visual.ImageStim(win, image = my_directory+"/correcte respons.jpg")
Spacebar_text = visual.TextStim(win, text='Press the space bar to continue', pos=(0,-0.8), color='black')
image_stim.draw()
Spacebar_text.draw()
win.flip()
event.waitKeys(keyList = "space")

## Announce what block is about to start
Block_start.draw()
win.flip()
event.waitKeys(keyList = "space")

## constants ##

## number of blocks and repetitions ##
conditions = 21
Ntrials = 144  

## initialize clock ##
my_clock        = core.Clock()

## graphical elements ##
cross=[
        (-0.1, +0.5),  # up
        (+0.1, +0.5),
        (+0.1, +0.1),
        (+0.5, +0.1),  # right
        (+0.5, -0.1),
        (+0.1, -0.1),
        (+0.1, -0.5),  # down
        (-0.1, -0.5),
        (-0.1, -0.1),
        (-0.5, -0.1),  # left
        (-0.5, +0.1),
        (-0.1, +0.1),
    ]
# MessageOnScreen = visual.TextStim(win,text="") #
grating = visual.GratingStim(
    win=win,
    units="pix",
    size=[240, 240]
)
grating.mask = "gauss"
orientation_hor=0.0
orientation_vert=90.0
grating_hpos = []
grating_vpos=[]
r=440
slicec=2*math.pi/12
for i in range(12):
    grating_hpos.append(r*math.sin(slicec*i))
    grating_vpos.append(r*math.cos(slicec*i))
print(grating_hpos)
print(grating_vpos)
contrast = 0.75
current=time.time()
win.flip()

kb = keyboard.Keyboard()
for condition_width in range(conditions):
    target_location=list()
    counter=0
    for i in range(Ntrials):
        target_location.append(counter)
        counter+=1
        counter=counter%12
    random.shuffle(target_location)

    for trial in range(Ntrials):
        current_orient=random.randint(0,1)==1
        for i_grating in range(12):
            if i_grating is target_location[trial]:
                # this patch will be the target #
                    
                grating.opacity = contrast
                grating.sf=(2-(0.05*condition_width))/30
                #grating.sf=(0.2-(0.005*condition_width)) #
                grating.ori = orientation_hor if current_orient else orientation_vert
                grating.pos = [grating_hpos[i_grating], grating_vpos[i_grating]]
            else:
                # this patch will not be the target #
                grating.opacity = contrast
                grating.sf = 2/30
                grating.ori = orientation_vert if current_orient else orientation_hor  
                grating.pos = [grating_hpos[i_grating], grating_vpos[i_grating]]

            grating.draw()
        text = visual.shape.ShapeStim(
            win=win,
            vertices=cross,
            units="pix",
            size=100,
            fillColor=[-1, -1, -1],
            lineColor=[-1, -1, -1]
        )

        text.draw()
        win.flip()
        print(target_location[trial])
        kb.clock.reset() 
        event.waitKeys()
        while True:
            keys = kb.getKeys(['right', 'left','up','down', 'quit','escape'], waitRelease=True)
            if 'quit' in keys or 'escape' in keys:
                core.quit()
            if len(keys)>0:
                key=keys[0]
                thisExp.addData('resp.key', key.name)
                thisExp.addData('resp.rt', key.rt)
                thisExp.addData('resp.duration',key.duration)
                
                correct=""
                if target_location[trial] in [11,0,1]:
                    correct='up'
                elif target_location[trial] in [2,3,4]:
                    correct='right'
                elif target_location[trial] in [5,6,7]:
                    correct='down'
                elif target_location[trial] in [8,9,10]:
                    correct='left'    
                    
                thisExp.addData('correct',correct==key.name )
                thisExp.addData('correct_position',target_location[trial])
                thisExp.addData('orient_tar','hor' if current_orient else 'ver')
            
                thisExp.addData('condition', str(2-(0.05*condition_width)))
                thisExp.nextEntry()
                print(key.name, key.rt, key.duration)
                break;
            time.sleep(0.05)
    text = visual.TextStim(
            win=win,
            text='Take 2 minutes pause, press space bar to continue.',
            rgb=(0,0,0)
        )
    text.draw()
    win.flip()
    event.waitKeys()
    while True:
        keys = kb.getKeys(['space'], waitRelease=True)
        if 'space' in keys :
            break
        time.sleep(0.05)

## adieu participant ##

## messages and stimuli ##
Goodbye         = visual.TextStim(win, text = "Thanks for participating! Alert the researcher ", color = "black")

## Display the goodbye messge ##
Goodbye.draw()
win.flip()
event.waitKeys(keyList = "space")

## close window ##
win.close()